@@ -1,12 +1,12 @@
 /*
- *  $Id: configfile.c,v 4.2 2005/02/13 01:48:00 bkorb Exp $
- *  Time-stamp:      "2005-07-26 10:07:26 bkorb"
+ *  $Id: configfile.c,v 1.19 2007/01/18 05:32:13 bkorb Exp $
+ *  Time-stamp:      "2007-01-13 12:49:10 bkorb"
  *
  *  configuration/rc/ini file handling.
  */
 
 /*
- *  Automated Options copyright 1992-2005 Bruce Korb
+ *  Automated Options copyright 1992-2006 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -53,7 +53,7 @@
 static void
 filePreset(
     tOptions*     pOpts,
-    const char*   pzFileName,
+    char const*   pzFileName,
     int           direction );
 
 static char*
@@ -113,7 +113,7 @@ skipUnknown( char* pzText );
 /*=export_func  configFileLoad
  *
  * what:  parse a configuration file
- * arg:   + const char*     + pzFile + the file to load +
+ * arg:   + char const*     + pzFile + the file to load +
  *
  * ret_type:  const tOptionValue*
  * ret_desc:  An allocated, compound value structure
@@ -141,24 +141,29 @@ skipUnknown( char* pzText );
  *  @end itemize
 =*/
 const tOptionValue*
-configFileLoad( const char* pzFile )
+configFileLoad( char const* pzFile )
 {
     tmap_info_t   cfgfile;
     tOptionValue* pRes = NULL;
+    tOptionLoadMode save_mode = option_load_mode;
+
     char* pzText =
         text_mmap( pzFile, PROT_READ, MAP_PRIVATE, &cfgfile );
 
-    if (pzText == MAP_FAILED)
+    if (TEXT_MMAP_FAILED_ADDR(pzText))
         return NULL; /* errno is set */
 
-    pRes = optionLoadNested(pzText, pzFile, strlen(pzFile), OPTION_LOAD_COOKED);
- all_done:
+    option_load_mode = OPTION_LOAD_COOKED;
+    pRes = optionLoadNested(pzText, pzFile, strlen(pzFile));
+
     if (pRes == NULL) {
         int err = errno;
         text_munmap( &cfgfile );
         errno = err;
     } else
         text_munmap( &cfgfile );
+
+    option_load_mode = save_mode;
     return pRes;
 }
 
@@ -167,8 +172,8 @@ configFileLoad( const char* pzFile )
  *
  * what:  find a hierarcicaly valued option instance
  * arg:   + const tOptDesc* + pOptDesc + an option with a nested arg type +
- * arg:   + const char*     + name     + name of value to find +
- * arg:   + const char*     + value    + the matching value    +
+ * arg:   + char const*     + name     + name of value to find +
+ * arg:   + char const*     + value    + the matching value    +
  *
  * ret_type:  const tOptionValue*
  * ret_desc:  a compound value structure
@@ -189,7 +194,7 @@ configFileLoad( const char* pzFile )
 =*/
 const tOptionValue*
 optionFindValue( const tOptDesc* pOptDesc,
-                 const char* pzName, const char* pzVal )
+                 char const* pzName, char const* pzVal )
 {
     const tOptionValue* pRes = NULL;
 
@@ -204,9 +209,8 @@ optionFindValue( const tOptDesc* pOptDesc,
 
     else do {
         tArgList* pAL = pOptDesc->optCookie;
-        int ct = pAL->useCt;
-        const tOptionValue** ppOV =
-            (const tOptionValue**)(void*)&(pAL->apzArgs);
+        int    ct   = pAL->useCt;
+        void** ppOV = (void**)(pAL->apzArgs);
 
         if (ct == 0) {
             errno = ENOENT;
@@ -214,7 +218,7 @@ optionFindValue( const tOptDesc* pOptDesc,
         }
 
         if (pzName == NULL) {
-            pRes = *ppOV;
+            pRes = (tOptionValue*)*ppOV;
             break;
         }
 
@@ -243,8 +247,8 @@ optionFindValue( const tOptDesc* pOptDesc,
  * what:  find a hierarcicaly valued option instance
  * arg:   + const tOptDesc* + pOptDesc + an option with a nested arg type +
  * arg:   + const tOptionValue* + pPrevVal + the last entry +
- * arg:   + const char*     + name     + name of value to find +
- * arg:   + const char*     + value    + the matching value    +
+ * arg:   + char const*     + name     + name of value to find +
+ * arg:   + char const*     + value    + the matching value    +
  *
  * ret_type:  const tOptionValue*
  * ret_desc:  a compound value structure
@@ -266,7 +270,7 @@ optionFindValue( const tOptDesc* pOptDesc,
 =*/
 const tOptionValue*
 optionFindNextValue( const tOptDesc* pOptDesc, const tOptionValue* pPrevVal,
-                 const char* pzName, const char* pzVal )
+                 char const* pzName, char const* pzVal )
 {
     int foundOldVal = 0;
     tOptionValue* pRes = NULL;
@@ -282,8 +286,8 @@ optionFindNextValue( const tOptDesc* pOptDesc, const tOptionValue* pPrevVal,
 
     else do {
         tArgList* pAL = pOptDesc->optCookie;
-        int ct = pAL->useCt;
-        tOptionValue** ppOV = (tOptionValue**)(void*)&(pAL->apzArgs);
+        int    ct   = pAL->useCt;
+        void** ppOV = (void**)pAL->apzArgs;
 
         if (ct == 0) {
             errno = ENOENT;
@@ -311,7 +315,7 @@ optionFindNextValue( const tOptDesc* pOptDesc, const tOptionValue* pPrevVal,
  *
  * what:  get a specific value from a hierarcical list
  * arg:   + const tOptionValue* + pOptValue + a hierarchcal value +
- * arg:   + const char*   + valueName + name of value to get +
+ * arg:   + char const*   + valueName + name of value to get +
  *
  * ret_type:  const tOptionValue*
  * ret_desc:  a compound value structure
@@ -333,7 +337,7 @@ optionFindNextValue( const tOptDesc* pOptDesc, const tOptionValue* pPrevVal,
  *  @end itemize
 =*/
 const tOptionValue*
-optionGetValue( const tOptionValue* pOld, const char* pzValName )
+optionGetValue( const tOptionValue* pOld, char const* pzValName )
 {
     tArgList*     pAL;
     tOptionValue* pRes = NULL;
@@ -345,11 +349,11 @@ optionGetValue( const tOptionValue* pOld, const char* pzValName )
     pAL = pOld->v.nestVal;
 
     if (pAL->useCt > 0) {
-        int ct = pAL->useCt;
-        tOptionValue** papOV = (tOptionValue**)(pAL->apzArgs);
+        int    ct    = pAL->useCt;
+        void** papOV = (void**)(pAL->apzArgs);
 
         if (pzValName == NULL) {
-            pRes = *papOV;
+            pRes = (tOptionValue*)*papOV;
         }
 
         else do {
@@ -393,8 +397,8 @@ optionGetValue( const tOptionValue* pOld, const char* pzValName )
  *  @code{ENOENT} - the supplied @code{pOldValue} pointed to the last entry.
  *  @end itemize
 =*/
-const tOptionValue*
-optionNextValue( const tOptionValue* pOVList, const tOptionValue* pOldOV )
+tOptionValue const *
+optionNextValue(tOptionValue const * pOVList,tOptionValue const * pOldOV )
 {
     tArgList*     pAL;
     tOptionValue* pRes = NULL;
@@ -406,8 +410,8 @@ optionNextValue( const tOptionValue* pOVList, const tOptionValue* pOldOV )
     }
     pAL = pOVList->v.nestVal;
     {
-        int   ct   = pAL->useCt;
-        tOptionValue** papNV = (tOptionValue**)(pAL->apzArgs);
+        int    ct    = pAL->useCt;
+        void** papNV = (void**)(pAL->apzArgs);
 
         while (ct-- > 0) {
             tOptionValue* pNV = *(papNV++);
@@ -417,7 +421,7 @@ optionNextValue( const tOptionValue* pOVList, const tOptionValue* pOldOV )
 
                 } else {
                     err  = 0;
-                    pRes = *papNV;
+                    pRes = (tOptionValue*)*papNV;
                 }
                 break;
             }
@@ -436,16 +440,15 @@ optionNextValue( const tOptionValue* pOVList, const tOptionValue* pOldOV )
 static void
 filePreset(
     tOptions*     pOpts,
-    const char*   pzFileName,
+    char const*   pzFileName,
     int           direction )
 {
     tmap_info_t   cfgfile;
+    tOptState     st = OPTSTATE_INITIALIZER(PRESET);
     char*         pzFileText =
         text_mmap( pzFileName, PROT_READ|PROT_WRITE, MAP_PRIVATE, &cfgfile );
-    char*         pzEndText;
-    tOptState     st = OPTSTATE_INITIALIZER(PRESET);
 
-    if (pzFileText == MAP_FAILED)
+    if (TEXT_MMAP_FAILED_ADDR(pzFileText))
         return;
 
     if (direction == DIRECTION_CALLED) {
@@ -462,17 +465,15 @@ filePreset(
     if ((pOpts->fOptSet & OPTPROC_PRESETTING) == 0)
         st.flags = OPTST_SET;
 
-    pzEndText = pzFileText + cfgfile.txt_size;
-
     do  {
-        while (isspace( *pzFileText ))  pzFileText++;
+        while (isspace( (int)*pzFileText ))  pzFileText++;
 
-        if (isalpha( *pzFileText )) {
+        if (isalpha( (int)*pzFileText )) {
             pzFileText = handleConfig( pOpts, &st, pzFileText, direction );
 
         } else switch (*pzFileText) {
         case '<':
-            if (isalpha( pzFileText[1] ))
+            if (isalpha( (int)pzFileText[1] ))
                 pzFileText = handleStructure(pOpts, &st, pzFileText, direction);
 
             else switch (pzFileText[1]) {
@@ -545,8 +546,11 @@ handleConfig(
     char* pzName = pzText++;
     char* pzEnd  = strchr( pzText, '\n' );
 
-    while (ISNAMECHAR( *pzText ))  pzText++;
-    while (isspace( *pzText )) pzText++;
+    if (pzEnd == NULL)
+        return pzText + strlen(pzText);
+
+    while (ISNAMECHAR( (int)*pzText ))  pzText++;
+    while (isspace( (int)*pzText )) pzText++;
     if (pzText > pzEnd) {
     name_only:
         *pzEnd++ = NUL;
@@ -560,10 +564,10 @@ handleConfig(
      *  is an invalid format and we give up parsing the text.
      */
     if ((*pzText == '=') || (*pzText == ':')) {
-        while (isspace( *++pzText ))   ;
+        while (isspace( (int)*++pzText ))   ;
         if (pzText > pzEnd)
             goto name_only;
-    } else if (! isspace(pzText[-1]))
+    } else if (! isspace((int)pzText[-1]))
         return NULL;
 
     /*
@@ -627,7 +631,7 @@ handleDirective(
     size_t name_len;
 
     if (  (strncmp( pzText+2, zProg, title_len ) != 0)
-       || (! isspace( pzText[title_len+2] )) )  {
+       || (! isspace( (int)pzText[title_len+2] )) )  {
         pzText = strchr( pzText+2, '>' );
         if (pzText != NULL)
             pzText++;
@@ -641,9 +645,9 @@ handleDirective(
     do  {
         pzText += title_len;
 
-        if (isspace(*pzText)) {
-            while (isspace(*pzText))  pzText++;
-            if (  (strneqvcmp( pzText, pOpts->pzProgName, name_len ) == 0)
+        if (isspace((int)*pzText)) {
+            while (isspace((int)*pzText))  pzText++;
+            if (  (strneqvcmp( pzText, pOpts->pzProgName, (int)name_len) == 0)
                && (pzText[name_len] == '>'))  {
                 pzText += name_len + 1;
                 break;
@@ -701,12 +705,12 @@ handleStructure(
     char*         pzText,
     int           direction )
 {
-    tOptionLoadMode mode = OPTION_LOAD_UNCOOKED;
+    tOptionLoadMode mode = option_load_mode;
     tOptionValue     valu;
 
     char* pzName = ++pzText;
+    char* pzData;
     char* pcNulPoint;
-    char* pzValStart;
 
     while (ISNAMECHAR( *pzText ))  pzText++;
     pcNulPoint = pzText;
@@ -720,13 +724,14 @@ handleStructure(
             break;
         if (*pzText != '/')
             return NULL;
+        /* FALLTHROUGH */
 
     case '/':
         if (pzText[1] != '>')
             return NULL;
         *pzText = NUL;
         pzText += 2;
-        loadOptionLine( pOpts, pOS, pzName, direction, OPTION_LOAD_KEEP );
+        loadOptionLine( pOpts, pOS, pzName, direction, mode );
         return pzText;
 
     case '>':
@@ -738,13 +743,13 @@ handleStructure(
             pzText++;
         return pzText;
     }
-    pzValStart = ++pzText;
 
     /*
-     *  If we are here, we have a value.  Separate the name from the
-     *  value for a moment.
+     *  If we are here, we have a value.  "pzText" points to a closing angle
+     *  bracket.  Separate the name from the value for a moment.
      */
     *pcNulPoint = NUL;
+    pzData = ++pzText;
 
     /*
      *  Find the end of the option text and NUL terminate it
@@ -758,7 +763,7 @@ handleStructure(
         sprintf( pz, "</%s>", pzName );
         *pzText = ' ';
         pzText = strstr( pzText, pz );
-        if (pz != z) free(pz);
+        if (pz != z) AGFREE(pz);
 
         if (pzText == NULL)
             return pzText;
@@ -770,8 +775,9 @@ handleStructure(
 
     /*
      *  Rejoin the name and value for parsing by "loadOptionLine()".
+     *  Erase any attributes parsed by "parseAttributes()".
      */
-    *(pcNulPoint++) = ' ';
+    memset(pcNulPoint, ' ', pzData - pcNulPoint);
 
     /*
      *  "pzName" points to what looks like text for one option/configurable.
@@ -794,7 +800,7 @@ internalFileLoad( tOptions* pOpts )
 {
     int     idx;
     int     inc = DIRECTION_PRESET;
-    char    zFileName[ MAXPATHLEN+1 ];
+    char    zFileName[ AG_PATH_MAX+1 ];
 
     if (pOpts->papzHomeList == NULL)
         return;
@@ -832,7 +838,7 @@ internalFileLoad( tOptions* pOpts )
 
         idx += inc;
 
-        if (! optionMakePath( zFileName, sizeof( zFileName ),
+        if (! optionMakePath( zFileName, (int)sizeof(zFileName),
                               pzPath, pOpts->pzProgPath ))
             continue;
 
@@ -852,8 +858,8 @@ internalFileLoad( tOptions* pOpts )
                 continue;
 
             pz = zFileName + len;
-            if (pz[-1] != '/')
-                *(pz++) = '/';
+            if (pz[-1] != DIRCH)
+                *(pz++) = DIRCH;
             strcpy( pz, pOpts->pzRcName );
         }
 
@@ -876,7 +882,7 @@ internalFileLoad( tOptions* pOpts )
  * what: Load the locatable config files, in order
  *
  * arg:  + tOptions*   + pOpts  + program options descriptor +
- * arg:  + const char* + pzProg + program name +
+ * arg:  + char const* + pzProg + program name +
  *
  * ret_type:  int
  * ret_desc:  0 -> SUCCESS, -1 -> FAILURE
@@ -905,7 +911,7 @@ internalFileLoad( tOptions* pOpts )
  *       always be returned.
 =*/
 int
-optionFileLoad( tOptions* pOpts, const char* pzProgram )
+optionFileLoad( tOptions* pOpts, char const* pzProgram )
 {
     if (! SUCCESSFUL( validateOptionsStruct( pOpts, pzProgram )))
         return -1;
@@ -924,7 +930,8 @@ optionFileLoad( tOptions* pOpts, const char* pzProgram )
  * arg:   + tOptDesc* + pOptDesc + the descriptor for this arg +
  *
  * doc:
- *  Processes the options found in the file named with pOptDesc->pzLastArg.
+ *  Processes the options found in the file named with
+ *  pOptDesc->optArg.argString.
 =*/
 void
 optionLoadOpt( tOptions* pOpts, tOptDesc* pOptDesc )
@@ -938,12 +945,12 @@ optionLoadOpt( tOptions* pOpts, tOptDesc* pOptDesc )
      */
     if (! DISABLED_OPT( pOptDesc )) {
         struct stat sb;
-        if (stat( pOptDesc->pzLastArg, &sb ) != 0) {
+        if (stat( pOptDesc->optArg.argString, &sb ) != 0) {
             if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
                 return;
 
             fprintf( stderr, zFSErrOptLoad, errno, strerror( errno ),
-                     pOptDesc->pzLastArg );
+                     pOptDesc->optArg.argString );
             (*pOpts->pUsageProc)( pOpts, EXIT_FAILURE );
             /* NOT REACHED */
         }
@@ -952,12 +959,12 @@ optionLoadOpt( tOptions* pOpts, tOptDesc* pOptDesc )
             if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
                 return;
 
-            fprintf( stderr, zNotFile, pOptDesc->pzLastArg );
+            fprintf( stderr, zNotFile, pOptDesc->optArg.argString );
             (*pOpts->pUsageProc)( pOpts, EXIT_FAILURE );
             /* NOT REACHED */
         }
 
-        filePreset(pOpts, pOptDesc->pzLastArg, DIRECTION_CALLED);
+        filePreset(pOpts, pOptDesc->optArg.argString, DIRECTION_CALLED);
     }
 }
 
@@ -994,7 +1001,7 @@ parseAttributes(
             break;
         }
 
-        while (isspace( *++pzText ))   ;
+        while (isspace( (int)*++pzText ))   ;
 
         if (strncmp( pzText, zLoadType, lenLoadType ) == 0) {
             pzText = parseValueType( pzText+lenLoadType, pType );
@@ -1049,7 +1056,7 @@ parseLoadMode(
         if (strncmp( pzText, zLoadCooked, len ) == 0) {
             if (  (pzText[len] == '>')
                || (pzText[len] == '/')
-               || isspace(pzText[len])) {
+               || isspace((int)pzText[len])) {
                 *pMode = OPTION_LOAD_COOKED;
                 return pzText + len;
             }
@@ -1062,7 +1069,7 @@ parseLoadMode(
         if (strncmp( pzText, zLoadUncooked, len ) == 0) {
             if (  (pzText[len] == '>')
                || (pzText[len] == '/')
-               || isspace(pzText[len])) {
+               || isspace((int)pzText[len])) {
                 *pMode = OPTION_LOAD_UNCOOKED;
                 return pzText + len;
             }
@@ -1075,7 +1082,7 @@ parseLoadMode(
         if (strncmp( pzText, zLoadKeep, len ) == 0) {
             if (  (pzText[len] == '>')
                || (pzText[len] == '/')
-               || isspace(pzText[len])) {
+               || isspace((int)pzText[len])) {
                 *pMode = OPTION_LOAD_KEEP;
                 return pzText + len;
             }
@@ -1116,7 +1123,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeString);
         if (strncmp( pzText, zLtypeString, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_STRING;
                 return pzText + len;
             }
@@ -1127,7 +1134,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeInteger);
         if (strncmp( pzText, zLtypeInteger, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_NUMERIC;
                 return pzText + len;
             }
@@ -1149,7 +1156,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeKeyword);
         if (strncmp( pzText, zLtypeKeyword, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_ENUMERATION;
                 return pzText + len;
             }
@@ -1160,7 +1167,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeSetMembership);
         if (strncmp( pzText, zLtypeSetMembership, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_MEMBERSHIP;
                 return pzText + len;
             }
@@ -1171,7 +1178,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeNest);
         if (strncmp( pzText, zLtypeNest, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_HIERARCHY;
                 return pzText + len;
             }
@@ -1193,7 +1200,7 @@ static char*
 skipUnknown( char* pzText )
 {
     for (;; pzText++) {
-        if (isspace( *pzText ))  return pzText;
+        if (isspace( (int)*pzText ))  return pzText;
         switch (*pzText) {
         case NUL: return NULL;
         case '/':
@@ -1212,7 +1219,7 @@ skipUnknown( char* pzText )
  *  already been called.)
  */
 LOCAL tSuccess
-validateOptionsStruct( tOptions* pOpts, const char* pzProgram )
+validateOptionsStruct( tOptions* pOpts, char const* pzProgram )
 {
     if (pOpts == NULL) {
         fputs( zAO_Bad, stderr );
@@ -1254,7 +1261,7 @@ validateOptionsStruct( tOptions* pOpts, const char* pzProgram )
      *  and the set of equivalent characters.
      */
     if (pOpts->pzProgName == NULL) {
-        const char* pz = strrchr( pzProgram, '/' );
+        char const* pz = strrchr( pzProgram, DIRCH );
 
         if (pz == NULL)
              pOpts->pzProgName = pzProgram;
@@ -1276,7 +1283,6 @@ validateOptionsStruct( tOptions* pOpts, const char* pzProgram )
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
- * tab-width: 4
  * indent-tabs-mode: nil
  * End:
  * end of autoopts/configfile.c */
