@@ -1,19 +1,29 @@
 /* -*- buffer-read-only: t -*- vi: set ro:
  *
  * Prototypes for autoopts
- * Generated Sun Oct  2 11:09:26 PDT 2005
+ * Generated Sat Jan 27 07:00:54 PST 2007
  */
 #ifndef AUTOOPTS_PROTO_H_GUARD
-#define AUTOOPTS_PROTO_H_GUARD
+#define AUTOOPTS_PROTO_H_GUARD 1
 #ifndef LOCAL
 #  define LOCAL extern
 #  define REDEF_LOCAL 1
 #else
 #  undef  REDEF_LOCAL
 #endif
-/*
- *  Extracted from autoopts.c
- */
+/*\n *  Extracted from autoopts.c\n */
+LOCAL void *
+ao_malloc( size_t sz );
+
+LOCAL void *
+ao_realloc( void *p, size_t sz );
+
+LOCAL void
+ao_free( void *p );
+
+LOCAL char *
+ao_strdup( char const *str );
+
 LOCAL tSuccess
 handleOption( tOptions* pOpts, tOptState* pOptState );
 
@@ -21,7 +31,7 @@ LOCAL tSuccess
 longOptionFind( tOptions* pOpts, char* pzOptName, tOptState* pOptState );
 
 LOCAL tSuccess
-shortOptionFind( tOptions* pOpts, tUC optValue, tOptState* pOptState );
+shortOptionFind( tOptions* pOpts, uint_t optValue, tOptState* pOptState );
 
 LOCAL tSuccess
 doImmediateOpts( tOptions* pOpts );
@@ -29,9 +39,7 @@ doImmediateOpts( tOptions* pOpts );
 LOCAL tSuccess
 doRegularOpts( tOptions* pOpts );
 
-/*
- *  Extracted from configfile.c
- */
+/*\n *  Extracted from configfile.c\n */
 LOCAL void
 internalFileLoad( tOptions* pOpts );
 
@@ -43,20 +51,16 @@ parseAttributes(
     tOptionValue*       pType );
 
 LOCAL tSuccess
-validateOptionsStruct( tOptions* pOpts, const char* pzProgram );
+validateOptionsStruct( tOptions* pOpts, char const* pzProgram );
 
-/*
- *  Extracted from environment.c
- */
+/*\n *  Extracted from environment.c\n */
 LOCAL void
 doPrognameEnv( tOptions* pOpts, teEnvPresetType type );
 
 LOCAL void
 doEnvPresets( tOptions* pOpts, teEnvPresetType type );
 
-/*
- *  Extracted from load.c
- */
+/*\n *  Extracted from load.c\n */
 LOCAL void
 mungeString( char* pzTxt, tOptionLoadMode mode );
 
@@ -68,15 +72,15 @@ loadOptionLine(
     tDirection  direction,
     tOptionLoadMode   load_mode );
 
-/*
- *  Extracted from sort.c
- */
+/*\n *  Extracted from nested.c\n */
+LOCAL tOptionValue*
+optionLoadNested(char const* pzTxt, char const* pzName, size_t nameLen);
+
+/*\n *  Extracted from sort.c\n */
 LOCAL void
 optionSort( tOptions* pOpts );
 
-/*
- *  Extracted from stack.c
- */
+/*\n *  Extracted from stack.c\n */
 LOCAL void
 addArgListEntry( void** ppAL, void* entry );
 
@@ -84,5 +88,4 @@ addArgListEntry( void** ppAL, void* entry );
 #  undef LOCAL
 #  define LOCAL
 #endif
-
 #endif /* AUTOOPTS_PROTO_H_GUARD */
